// file: calc.cpp
//   copyright  nakazato@phy.cam.ac.uk
//
//   some codes based on
//       N. Wirth, "Algorithms + Data Structurtes = Programs" Prentice-Hall inc,  1978
//       B. W. Kernigham and P. J. Plauger, "Software Tools" Bell Telephone Lab., 1976
//
//   1st Feb. 2003
//       first version
//   8th Feb. 2003
//       more error messages
//   15th Feb. 2003
//       'list' is added
//   23rd Feb. 2003
//       'clear' is added
//       'delTable' is added
//   10th Aug. 2003
//       while syntax is added

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include "calc.h"

#define pi   3.141592653589793238
#define lowerlimit 1e-13

enum { nul,     ident,     number, comma,
       plus,    minus,     times,  slash,  percent, lparen, rparen,
       eql,     period,    pw,     lss,    gtr,     fileend,
       sinsym,  cossym,   tansym,  asinsym, acossym,  atansym,
       intsym,  roundsym, expsym, sqrtsym,
       lnsym,   logsym,   abssym, pisym,  andsym,  orsym,
       eqlsym,  neqlsym,  notsym, ifsym,  thensym, elsesym,
       whilesym };

const int ssym[256] = {   //ASCII
   fileend,nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   percent, nul,    nul,
   lparen, rparen, times, plus, comma, minus,   period, slash,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  lss,   eql,     gtr,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     pw,     nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul,
   nul,    nul,    nul,   nul,  nul,   nul,     nul,    nul
};

const char *word[now] = {
   "",     "SIN",  "COS",  "TAN",
   "ASIN", "ACOS", "ATAN", "INT", "ROUND",
   "EXP",  "SQRT", "LN",   "LOG", "ABS",
   "PI",   "AND",  "OR",   "EQ",  "NE",
   "NOT",  "IF",   "THEN", "ELSE", "WHILE"
};

const int wsym[now] = {
   ident,   sinsym,  cossym,  tansym,
   asinsym, acossym, atansym, intsym,  roundsym,
   expsym,  sqrtsym, lnsym,   logsym,  abssym,
   pisym,   andsym,  orsym,   eqlsym,  neqlsym,
   notsym,  ifsym,   thensym, elsesym, whilesym
};

calc::calc()
{
   ang = 0;
   clear();
}

calc::~calc()
{
}

// interp error

const char *error[15] = {
   "no error",
   "table buffer overflow",
   "too many variables",
   "missing right paren",
   "missing left paren",
   "ln negative argument",
   "log negative argument",
   "divide by 0",
   "ident ?",
   "syntax error",
   "missing then",
   "sqrt negative argument",
   "out of range",
   "too many arguments",
   "number of arguments"
};

// error handle
void calc::error_(input *t, work *w, int i)
{
   if ((i) && (!((w->SyntaxC) &&
       ((i==5)||(i==6)||(i==7)||(i==11)||(i==12))))) {
      while (t->text[t->p]!=0) (t->p)++;
      t->ch  = 0;
      t->sym = fileend;
      if (w->error_no==0) w->error_no  = i;
   }
}

// remove comment and duplicated spaces
int calc::condense(char *s, int i)
{
   int j, k, n;

// skip comment
   j=k=i;
   while (s[j]) {
      if (s[j]=='{') {
         j++;
         while ((s[j]!='}')&&(s[j]!=0)) j++;
         if (s[j]=='}') j++;
      }
      if (s[j]==0) break;
      s[k++]=s[j++];
   }
   s[k++]=0;
// skip spaces
   j=k=i;
   while (isspace(s[j])) j++;
   while (s[j]) {
      n=0;
      while (isspace(s[j])) { j++, n++;}
      if (s[j]==0) break;
      if (n>0) s[k++]=' ';
      s[k++]=s[j++];
   }
   s[k++]=0;
   return k;
}

// get one character
void calc::getch(input *t)
{
   t->ch = t->text[t->p];
   if (t->ch) (t->p)++;
}

// isdigit (arbitrary base)
int calc::isdigitb(char c, int base)
{
   int i;

   i = -1;
   if (isdigit(c)) i = c - '0';
   if (isalpha(c)) i = toupper(c)-'A'+10;
   if (base>i) return i;
   return -1;
}

// constant value
//   unit can be attached as SPICE
double calc::constant(input *t)
{
   double x;
   int power,exp,sign,base,i;

   x=0.0;

   base=10;
   if (t->ch=='0'){
      getch(t);
      switch (toupper(t->ch)){
         case 'B':base= 2; getch(t); break;
         case 'O':base= 8; getch(t); break;
         case 'X':base=16; getch(t); break;
         case 'N':
            getch(t);
            base=0;
            while (isdigit(t->ch)){
               base=10*base+t->ch-'0';
               getch(t);
            }
            if (toupper(t->ch)!='N') return x;
            getch(t);
            break;
       }
   }

   while ((i=isdigitb(t->ch,base))>=0){
      x=base*x + i;
      getch(t);
   }
   power=0;
   if (t->ch == '.'){
      getch(t);
      while ((i=isdigitb(t->ch,base))>=0) {
         power--;
         x=base*x + i;
         getch(t);
      }
   }
   exp=0;
   if (toupper(t->ch) == 'E') {
      getch(t);
      sign=(t->ch == '-');
      if ((t->ch == '+')||(t->ch == '-')) getch(t);
      while ((i=isdigitb(t->ch,base))>=0) {
         exp=base*exp + i;
         getch(t);
      }
      if (sign) exp*=(-1);
   }
   exp+=power;
   switch (toupper(t->ch)) {
      case 'T':exp+=12; break;
      case 'G':exp+=9; break;
      case 'M':getch(t);
               if (toupper(t->ch)=='E') {
                  getch(t);
                  if (toupper(t->ch)=='G') exp+=6;else exp-=3;
               } else exp-=3;
               break;
      case 'K':exp+=3; break;
      case 'U':exp-=6; break;
      case 'N':exp-=9; break;
      case 'P':exp-=12; break;
      case 'F':exp-=15; break;
      case 'A':exp-=18; break;
   }
   while (isalpha(t->ch)) getch(t);
   if (exp!=0) x *= pow((double) base, (double) exp);
   return x;
}

void calc::getsym(input *t)
{
   int i,j;

   t->prevp = t->p - (t->ch?1:0);
   while (isspace(t->ch)) getch(t);
   if (isalpha(t->ch)) {
      i=0;
      do {
         if (i<al) t->id[i++]=t->ch;
         getch(t);
      } while (isalnum(t->ch));
      t->id[i]=0;
      for (i=now-1;i>0;i--) {
         for (j=0;(toupper(t->id[j])==word[i][j])&&(t->id[j]!=0);j++);
         if (t->id[j]==word[i][j]) break;
      }
      t->sym=wsym[i];
   } else if ((isdigit(t->ch))||(t->ch=='.')) {
      t->sym=number;
      t->num=constant(t);
   } else {
      t->sym=ssym[t->ch];
      getch(t);
   }
}

int calc::lookup(char *name, vtable *vtable)
{
   int i,j,k;

   for(i=0;i<vtable->tx;i++) {
      j=vtable->namptr[i];
      for (k=0;(toupper(name[k])==toupper(vtable->table[j]))&&(name[k]!=0);k++,j++);
      if (name[k]==vtable->table[j]) return i;
   }
   return vtable->tx;
}

void calc::instal(char *name, input *t, vtable *vtable, work *w)
{
   int nlen;

   nlen=strlen(name)+1;
   if (vtable->lastt+nlen>=tmax||vtable->tx>=txmax) error_(t, w, 1);
   else {
      vtable->namptr[vtable->tx]=vtable->lastt;
      (vtable->tx)++;
      strcpy(&vtable->table[vtable->lastt],name);
      (vtable->lastt)+=nlen;
   }
}

int calc::position(char *id, input *t, vtable *vtable, work *w)
{
   int i;

   i=lookup(id,vtable);
   if (i==vtable->tx) {
      if (vtable->tx<txmax) {
         if (!w->SyntaxC) instal(id, t, vtable, w);
         vtable->value[i] = 0.0;
         vtable->flag[i]  = 0;
         vtable->funcstr[i].n = 0;
         vtable->funcstr[i].exptr = vtable->exbufptr;
         vtable->exbuf[vtable->exbufptr] = 0;
      } else error_(t, w, 2);
   }
   return i;
}

double calc::getarg(input *t, vtable *v, work *w)
{
   double x;

   x = 0.0;
   if (t->sym==lparen) {
      getsym(t);
      do {
         x= expr(t, v, w);
         } while ((t->sym!=rparen)&&(t->sym!=fileend));
      if (t->sym==rparen) getsym(t); else error_(t, w, 3);
   } else error_(t, w, 4);
   return x;
}

double calc::fgetarg(input *t, vtable *v, work *w)
{
   if (ssym[t->ch]==lparen) {
       getsym(t);
       return getarg(t, v, w);
   } else {
       error_(t, w, 4);
       return 0.0;
   }
}

double calc::AngArg(double x)
{
   switch (ang) {
      case 1:
         x *= pi/180.0;
         break;
      default:
         ;
   }
   return x;
}

double calc::_AngArg(double x)
{
   switch (ang) {
      case 1:
         x /= pi/180.0;
         break;
      default:
         ;
   }
   return x;
}

double calc::value(input *t, vtable *v, work *w)
{
   double x, y, d[index_max], s[index_max];
   int    i, f, sign;
   struct input t1;

   x = 0.0;
   sign=(t->sym==minus);
   if ((t->sym==plus)||(t->sym==minus)) getsym(t);
   if (t->sym==lparen) x=getarg(t, v, w); else
   switch (t->sym) {
      case number:
         x=t->num;
         getsym(t);
         break;
      case sinsym:
         x=sin(AngArg(fgetarg(t, v, w)));
         break;  
      case cossym:
         x=cos(AngArg(fgetarg(t, v, w)));
         break;  
      case tansym:
         x=tan(AngArg(fgetarg(t, v, w)));
         break;  
      case asinsym:
         y=fgetarg(t, v, w);
         if (fabs(y) <= 1.0) x= _AngArg(asin(y)); else error_(t, w, 12);
         break;  
      case acossym:
         y=fgetarg(t, v, w);
         if (fabs(y) <= 1.0) x= _AngArg(acos(y)); else error_(t, w, 12);
         break;  
      case atansym:
         x= _AngArg(atan(fgetarg(t, v, w)));
         break;  
      case intsym:
         x=floor(fgetarg(t, v, w));
         break;  
      case roundsym:
         x=floor(fgetarg(t, v, w)+0.5);
         break;  
      case expsym:
         x=exp(fgetarg(t, v, w));
         break;  
      case sqrtsym:
         y=fgetarg(t, v, w);
         if (y>0) x=sqrt(y); else error_(t, w, 11);
         break;  
      case lnsym:
         y=fgetarg(t, v, w);
         if (y>0) x=log(y);  else error_(t, w, 5);
         break;  
      case logsym:
         y=fgetarg(t, v, w);
         if (y>0) x=log10(y); else error_(t, w, 6);
         break;  
      case abssym:
         x=fabs(fgetarg(t, v, w));
         break;  
      case pisym:
         getsym(t);
         x=pi;
         break;  
      case ident:
   // examine index
         x=v->value[f=position(t->id, t, v, w)];
         i=0;
         if (ssym[t->ch]==lparen) {
            getsym(t);
            do {
               getsym(t);
               d[i]=expr(t, v, w);
               wfuncstr[f].ptr[i] = w->ptr;
               if (i<index_max-1) i++; else error_(t, w, 13);
            } while(t->sym==comma);
            if (t->sym==rparen) getsym(t); else error_(t, w, 3);
         } else getsym(t);
         wfuncstr[f].n=i;
   // evaluation
         if ((t->sym!=eql)&&(!w->SyntaxC)) {
            if (v->funcstr[f].n!=wfuncstr[f].n) error_(t, w, 14);
            else if ((v->funcstr[f].n>0)&&(v->exbuf[v->funcstr[f].exptr]!=0) ) {
               for (i=0;i<v->funcstr[f].n;i++) {
                  s[i]=v->value[v->funcstr[f].ptr[i]];
                  v->value[v->funcstr[f].ptr[i]]=d[i];
               }
               t1.p=v->funcstr[f].exptr;
               t1.text=v->exbuf;
               getch(&t1);
               do getsym(&t1); while (t1.sym==nul);
               x=expr(&t1, v, w);
               for (i=0;i<v->funcstr[f].n;i++) v->value[v->funcstr[f].ptr[i]]=s[i];
            }
         }
         break;
      default: error_(t, w, 9);
   }
   if (sign) return -x; else return x;
}

double calc::_not(input *t, vtable *v, work *w)
{
   if (t->sym == notsym) {
      getsym(t);
      return ! _not(t, v, w);
   } else return value(t, v, w);
}

double calc::power(input *t, vtable *v, work *w)
{
   double x;

   x=_not(t, v, w);
   while (t->sym == pw) {
      getsym(t);
      x=pow(x, _not(t, v, w));
   }
   return x;
}

double calc::multidiv(input *t, vtable *v, work *w)
{
   double x,y;

   x = power(t, v, w);
   while ((t->sym == times)||(t->sym == slash)||(t->sym == percent))
      if (t->sym == times){
         getsym(t);
         x *= power(t, v, w);
      } else if (t->sym == slash) {
         getsym(t);
         y = power(t, v, w);
         if (y == 0.0) error_(t, w, 7); else x/=y;
      } else {
         getsym(t);
         y = power(t, v, w);
         if (y == 0.0) error_(t, w, 7); else x = fmod(x,y);
      }
   return x;
}

double calc::addsub(input *t, vtable *v, work *w)
{
   double x;

   x = multidiv(t, v, w);
   while ((t->sym == plus)||(t->sym == minus)){
      if (t->sym == plus){
         getsym(t);
         x += multidiv(t, v, w);
      } else {
         getsym(t);
         x -= multidiv(t, v, w);
      }
   }
   return x;
}

// boolean

double calc::gtrlss(input *t, vtable *v, work *w)
{
   double x, y;

   x=addsub(t, v, w);
   switch (t->sym){
      case gtr:
         getsym(t);
         if (t->sym == eql) {
            getsym(t);
            y=addsub(t, v, w);
            x= (x >= y);
         } else {
            y=addsub(t, v, w);
            x= (x > y);
         }
         break;
      case lss:
         getsym(t);
         if (t->sym == eql) {
            getsym(t);
            y=addsub(t, v, w);
            x= (x <= y);
         } else {
            y=addsub(t, v, w);
            x= (x < y);
         }
         break;
   }
   return x;
}

double calc::equal(input *t, vtable *v, work *w)
{
   double x, y;

   x=gtrlss(t, v, w);
   switch (t->sym){
      case eqlsym:
         getsym(t);
         y=gtrlss(t, v, w);
         x= ( fabs(x - y) <= lowerlimit*(fabs(x)+fabs(y)) );
         break;
      case neqlsym:
         getsym(t);
         y=gtrlss(t, v, w);
         x= ( fabs(x - y) >  lowerlimit*(fabs(x)+fabs(y)) );
         break;
   }
   return x;
}

double calc::_and(input *t, vtable *v, work *w)
{
   double x, y;

   x= equal(t, v, w);
   while (t->sym == andsym) {
      getsym(t);
      y= equal(t, v, w);
      x = x && y;
   }
   return x;
}

double calc::_or(input *t, vtable *v, work *w)
{
   double x, y;

   x= _and(t, v, w);
   while (t->sym == orsym) {
      getsym(t);
      y= _and(t, v, w);
      x= x || y;
   }
   return x;
}

double calc::ifthenelse(input *t, vtable *v, work *w)
{
   bool   f;
   char   ch;
   int    p;
   double x, y, z;
   work   ww;

   x=y=z=0.0;
   if (t->sym == ifsym) {
      getsym(t);
      x = _or(t, v, w);
      if (t->sym == thensym) {
         getsym(t);
         if ((!w->SyntaxC)&&(x))
            y = _or(t, v, w);
         else {
            ww.error_no = 0;
            ww.SyntaxC  = true;
            y = _or(t, v, &ww);
            error_(t, w, ww.error_no);
         }
         if (t->sym == elsesym) {
            getsym(t);
            if ((w->SyntaxC)||(x)) {
               ww.error_no = 0;
               ww.SyntaxC  = true;
               z = ifthenelse(t, v, &ww);
               error_(t, w, ww.error_no);
            } else
               z = ifthenelse(t, v, w);
         }
      } else error_(t, w, 10);
      return (x)?y:z;
   } else if (t->sym == whilesym) {
      p  = t->p;
      ch = t->ch;
      f  = true;
      while (f) {
         getsym(t);
         x = _or(t, v, w);
         while (t->sym == nul) getsym(t);
         if ((w->SyntaxC)||(! x)) {
            ww.error_no = 0;
            ww.SyntaxC  = true;
            y = _or(t, v, &ww);
            error_(t, w, ww.error_no);
            f = false;
         } else {
            x = _or(t, v, w);
            if (w->error_no==0) {
               t->p  = p;
               t->ch = ch;
            } else f=false;
         }
      }
      return x;
   } else return _or(t, v, w);
}

double calc::let(input *t, vtable *v, work *w)
{
   int    i, j, k, n, len;
   bool   f, f1;
   double x;
   work   ww;

   x = 0.0;
   if (t->sym == ident) {
      w->ptr = i = position(t->id, t, v, w);
      x = ifthenelse(t, v, w);
      if (t->sym == eql) {
         k = t->p-1;
         getsym(t);
         if (wfuncstr[i].n > 0) {
            ww.error_no = 0;
            ww.SyntaxC = true;
            let(t, v, &ww);
            error_(t, w, ww.error_no);
         } else {
            x = let(t, v, w);
            if (!w->SyntaxC) v->value[i] = x;
         }
   // registration
         if ((!w->SyntaxC)&&(w->error_no==0)) {
      // get expression
            if (wfuncstr[i].n > 0) {
               len = t->prevp - k;
               n = j = v->funcstr[i].exptr;
               while (v->exbuf[j]) j++;
               f = (j -n >=len);
               f1 = false;
               if (!f) {
                  n = v->funcstr[i].exptr = v->exbufptr;
                  f1 = (n+len+1 < index_maxc);
               }
               if (f||f1)
                  for(;k<t->prevp;) v->exbuf[n++] = t->text[k++];
               else error_(t, w, 1);
               v->exbuf[n] = 0;
               if (f1) v->exbufptr += len + 1;
            }
            v->flag[i] |= _ASSIGN;
      // set pointers
            v->funcstr[i].n = wfuncstr[i].n;
            for(k=0; k<wfuncstr[i].n; k++) {
               v->funcstr[i].ptr[k]=wfuncstr[i].ptr[k];
               v->flag[v->funcstr[i].ptr[k]] |= _FINDEX;
            }
         }
      }
   } else x=ifthenelse(t, v, w);
   return x;
}

//

double calc::expr(input *t, vtable *v, work *w)
{
   return let(t, v, w);
}

//********** public functions

// clear table
void calc::clear()
{
   v.exbufptr = 0;
   v.tx  = v.lastt  = 0;
}

// read table

int calc::read_table(char *name[], int *index)
{
   int i;

   for(i=0;i<v.tx;i++) {
      name[i]=&(v.table[v.namptr[i]]);
      if ((v.table[v.namptr[i]]) && (v.flag[i] & _ASSIGN))
         index[i]=v.funcstr[i].n;
      else
         index[i]= -1;
   }
   return v.tx;
}

//

void calc::delTable(int i)
{
   if ( (i < 0) || (i >= v.tx) ) return;
   if (v.flag[i] & _FINDEX) {
        v.flag[i] &= (! _ASSIGN);
        return;
   }
   v.table[v.namptr[i]] = 0;
}   

// list

#define addlistchar(c)  { if (n<maxchar-2) list[++n]=c; }

void calc::list(char *list, int maxchar)
{
   int i, j, k, m, n, index;
   char s[32];

   n = -1;
   for(i=0;i<v.tx;i++) {
      j = v.namptr[i];
      if ((v.table[j]) && (v.flag[i] & _ASSIGN)) {
         index=v.funcstr[i].n;
         while (v.table[j]) addlistchar(v.table[j++]);
         if (index) {
            addlistchar('(');
            for(k=0;k<index;k++){
               j = v.namptr[v.funcstr[i].ptr[k]];
               while (v.table[j]) addlistchar(v.table[j++]);
               addlistchar(',');
            }
            list[n]=')';
            addlistchar('=');
            j = v.funcstr[i].exptr;
            while (v.exbuf[j]) addlistchar(v.exbuf[j++]);
         } else {
            addlistchar('=');
            sprintf(s,"%17.12e",v.value[i]);
            m = -1;
            while(s[++m]!='e');
            while(s[--m]=='0');
            k=0;
            while(k<=m) addlistchar(s[k++]);
            while(s[k++]=='0');
            k--;
            while(s[k]) addlistchar(s[k++]);
         }
      }
      addlistchar('\n');
   }
   list[++n]=0;
}

//
#define MAXN 256
void calc::nconvert(char *s, double x, int n, int m)
{
   int ix,i,k,kmax,d[MAXN];
   double fx;
   char t[MAXN];

   i=0;
   if (x<0.0){
      s[i++]='-';
      x = (-x);
   }
   if (n!=10) {
      s[i++]='0';
      switch (n) {
         case  2:s[i++]='b'; break;
         case  8:s[i++]='o'; break;
         case 16:s[i++]='x'; break;
         default:
            s[i++]='n';
            sprintf(t,"%d",n);
            for(k=0;t[k]!=0;k++) s[i++]=t[k];
            s[i++]='n';
            break;
      }
   }

   ix=(int)x;
   fx=x-ix;
   for(k=m;k<MAXN;k++){
      d[k]=ix % n;
      ix /= n;
      if (ix==0) break;
   }
   if (k<MAXN-1) {
      kmax=k+1;
      d[kmax]=0;
   } else
      kmax=MAXN-1;
   if (m>0)
      for(k=m-1;k>=0;k--){
         d[k]=int(fx*n);
         fx=n*fx-d[k];
      }
   if (int(fx*n)>=n/2)
      for(k=0; k<=kmax; k++) {
         d[k]++;
         if (n>d[k]) break; else d[k]=0;
      }
   while (d[kmax]==0) kmax--;

   for(k=kmax;k>=m;k--)
      s[i++]=(d[k]<10)?('0'+d[k]):('a'+d[k]-10);
   if (m>0) {
      s[i++]='.';
      for(k=m-1;k>=0;k--)
         s[i++]=(d[k]<10)?('0'+d[k]):('a'+d[k]-10);
   }
   s[i]=0;  
}

void calc::sprintfn(char *s, char *f, double x)
{
   int i,i1,m,n;
   char c1;

   i1=(-1);
   n=10;
   i=0;
   while ((f[i]!='/')&&(f[i]!=0)) i++;
   if (f[i]=='/') {
      c1=f[i1=i];
      f[i]=0;
      i++;
      while (isspace(f[i])) i++;
      n=0;
      switch (toupper(f[i])) {
         case 'B':n= 2; break;
         case 'O':n= 8; break;
         case 'X':n=16; break;
         default:
            while (isdigit(f[i])) n=10*n+f[i++]-'0';
            break;
      }
   }
   if ((n==10)||(n<=0))
      sprintf(s, f, x);
   else {
      m=0;
      i=0;
      while ((f[i]!='.')&&(f[i]!=0)) i++;
      if (f[i]=='.') {
         i++;
         while (isdigit(f[i])) m=10*m+f[i++]-'0';
      }
      nconvert(s, x, n, m);
   }
   if (i1>=0) f[i1]=c1;
}

// main

int calc::result(char *text, double *x, char *result, char *format)
{
   input t;
   work  w;

   w.SyntaxC = false;
   w.error_no = 0;
   condense(text, 0);
   t.p=0;
   t.text=text;
   getch(&t);
   getsym(&t);
   if ( t.sym == fileend ) {
      *x = 0.0;
      result[0] = 0;
      return 0;
   }
   do {
      *x = 0.0;
      switch (t.sym) {
         case nul:
            getsym(&t);
            break;
         case elsesym:
         case thensym:
            error_(&t, &w, 10);
            break;
         case eql:
            error_(&t, &w, 9);
            break;
         default:
            *x=expr(&t, &v, &w);
      }
      if (w.error_no>0) break;
   } while (t.sym!=fileend);
   if (w.error_no>0)
      sprintf(result,"error:%s",error[w.error_no]);
   else {
      sprintfn(result,format,*x);
      v.value[position("Ans", &t, &v, &w)]=(*x);
   }
   return w.error_no;
}

void calc::set_ang(int i)
{
    switch (i) {
        case 1:
            ang = i;
            break;
        default:
            ang = 0;
    }
}
