// file slcalc.cpp
//   copyright  nakazato@phy.cam.ac.uk
//   1st Feb. 2003
//      first version
//   8th Feb. 2003
//      cut& paste buttons are added
//      slcalc::backspace is rewritten to avoid flashing
//      cursol positon and action to marked text are unified
//   15th Feb. 2003
//      menu (edit, options, help) is added
//   23rd Feb. 2003
//      add edit menu constants and functions
//      install keyboard input method
//   26th Feb. 2003
//      change qmenubar to qpemenubar
//   28th Feb. 2003
//      unify VGA, VGA view style, QVGA modes
//   30th July 2003
//      add options menu load file
//      SetConstName() is rewritten to reflect redefinition of constants & functions
//   4th Aug. 2003
//      garbage collection
//   10th Aug. 2003
//      while statement

#include "slcalc.h"
#include <string.h>
#include <ctype.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qradiobutton.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qtextbrowser.h>
#include <qlayout.h>

extern bool QVGA;
#define SLC700 (!QVGA)

// globals
calc   calc_;

slcalcSetting      * popupSetting;
slcalcList         * popupList;
slcalcEditConsFunc * popupEditConsFunc;
slcalcFileNameDiag * popupFileNameDiag;

//
int  g_bFormat   = 0;     // claculated result format n:0 f:1 e:2
int  g_bAng      = 0;     // angle base rad:0 deg:1
bool g_bClear    = true;  // clear input text at next key
int  g_bBufCount = 0;     // buffer count
double g_bX      = 0;     // calculated result
bool g_bError    = false; // make error ?
bool g_bCFedited = false; // conf file edited ?

//
char ws[MAXIFC];

// setting
const char * conf_file_name   = "/home/zaurus/Settings/slcalc.conf";

QString g_sFormatN = "%g";
QString g_sFormatF = "%f";
QString g_sFormatE = "%21.15e";
bool    g_sFlagBuffer     = true;
QString g_sBufferFileName = "/home/zaurus/Settings/slcalc.buffer";

slcalcSetting::slcalcSetting( QWidget* parent,  const char* name )
    : slcalcSet( parent, name, WType_Popup )
{
    connect(PushButtonOK,     SIGNAL(clicked()), this, SLOT(buttonOK()));
    connect(PushButtonCancel, SIGNAL(clicked()), this, SLOT(buttonCancel()));
}

slcalcSetting::~slcalcSetting()
{
}

void slcalcSetting::set(slcalc * parent)
{
    popupParent = parent;
    LineEditN->setText(g_sFormatN);
    LineEditF->setText(g_sFormatF);
    LineEditE->setText(g_sFormatE);
    CheckBoxBF->setChecked(g_sFlagBuffer);
    LineEditBFname->setText(g_sBufferFileName);
    g_bCFedited = false;
}

void slcalcSetting::buttonOK()
{
    if (strcmp(g_sFormatN, LineEditN->text())!=0) {
        g_bCFedited = true;
        g_sFormatN  = LineEditN->text();
    }
    if (strcmp(g_sFormatF, LineEditF->text())!=0) {
        g_bCFedited = true;
        g_sFormatF = LineEditF->text();
    }
    if (strcmp(g_sFormatE, LineEditE->text())!=0) {
        g_bCFedited = true;
        g_sFormatE = LineEditE->text();
    }
    if (g_sFlagBuffer != CheckBoxBF->isChecked()) {
        g_bCFedited  = true;
        g_sFlagBuffer = CheckBoxBF->isChecked();
    }
    if (strcmp(g_sBufferFileName, LineEditBFname->text())!=0) {
        g_bCFedited = true;
        g_sBufferFileName = LineEditBFname->text();
    }
    buttonCancel();
}

void slcalcSetting::buttonCancel()
{
    popupParent->MultiLineEdit1->setFocus();
    close();
}

// List

slcalcList::slcalcList( QWidget* parent,  const char* name )
    : slcalcL( parent, name, WType_Popup )
{
    connect(TextBrowser1,SIGNAL(ReturnPressed()), this, SLOT(closelist()));
}

slcalcList::~slcalcList()
{
}

void slcalcList::list(slcalc * parent, char *s)
{
    popupParent = parent;
    TextBrowser1->setText(s);
}

void slcalcList::closelist()
{
    popupParent->MultiLineEdit1->setFocus();
    close();
}

// get file name

slcalcFileNameDiag::slcalcFileNameDiag( QWidget* parent,  const char* name )
    : getFileNameDiag( parent, name, WType_Popup )
{
    connect(ListBox1,SIGNAL(OK()), this, SLOT(OK_selected()));
    connect(ListBox1,SIGNAL(Cancel()), this, SLOT(Cancel_selected()));
    connect(PushButtonOK, SIGNAL(clicked()), this, SLOT(OK_selected()) );
    connect(PushButtonCancel, SIGNAL(clicked()), this, SLOT(Cancel_selected()) );
}

slcalcFileNameDiag::~slcalcFileNameDiag()
{
}

void slcalcFileNameDiag::list(slcalc * parent, char *s)
{
    popupParent = parent;
    dir(s);
}

void slcalcFileNameDiag::OK_selected()
{
    QString Filename = ListBox1->currentText();
    if ((Filename!="")&&(Filename!="..")&&(Filename.contains("/")!=1))
        popupParent->FileLoad(DirName+"/"+Filename);
    popupParent->MultiLineEdit1->setFocus();
    close();
}

void slcalcFileNameDiag::Cancel_selected()
{
    popupParent->MultiLineEdit1->setFocus();
    close();
}

// Edit Constants and Functions

slcalcEditConsFunc::slcalcEditConsFunc( QWidget* parent,  const char* name )
    : slcalcECF( parent, name, WType_Popup )
{
    connect(PushButtonOK,     SIGNAL(clicked()), this, SLOT(buttonOK()));
    connect(PushButtonCancel, SIGNAL(clicked()), this, SLOT(buttonCancel()));
    connect(PushButtonPrev,   SIGNAL(clicked()), this, SLOT(buttonPrev()));
    connect(PushButtonNext,   SIGNAL(clicked()), this, SLOT(buttonNext()));

    listno = 0;
}

slcalcEditConsFunc::~slcalcEditConsFunc()
{
}

void slcalcEditConsFunc::buttonOK()
{
    char tx[MAXRC];
    int i;
    double x;

    getn(listno);
    calc_.clear();
    for (i = 0; i < MAXNN ; i++)
        if (!check[i]) calc_.result(line[i], &x, tx);
    popupParent->resetList();
    buttonCancel();
}

void slcalcEditConsFunc::buttonCancel()
{
    popupParent->MultiLineEdit1->setFocus();
    close();
}

void slcalcEditConsFunc::buttonPrev()
{
    if ( listno - 10 < 0) return;
    getn(listno);
    listno -= 10;
    setn(listno);
}

void slcalcEditConsFunc::buttonNext()
{
    if ( listno + 10 >= MAXNN) return;
    getn(listno);
    listno += 10;
    setn(listno);
}

void slcalcEditConsFunc::setn(int n)
{
    int i;
    char s[MAXSC];

    for (i = 0 ; i < 10 ; i++) {
        CheckBox[i]->setChecked(check[n+i]);
        LineEdit[i]->setText(line[n+i]);
        LineEdit[i]->home (false);
    }

    sprintf(s, "%d - %d", n + 1, n + 10);
    LineEdit0->setText(s);
}

void slcalcEditConsFunc::set(slcalc * parent)
{
    int i, j, n;

    popupParent = parent;
    for (i = 0 ; i < MAXNN ; i++)
        check[i] = edited[i] = false;
    line[0] = &(ws[0]);
    n = 1;
    for(i = 0; ws[i]; i++)
        if (ws[i]=='\n') {
            ws[i] = 0;
            line[n++] = &(ws[i + 1]);
        }
    ptr = i + 1;
    for(i = j = 0 ; i < n ; i++) 
        if (line[i][0]) {
            line[j]  = line[i];
            index[j] = i;
            j++;
        }
    ws[MAXIFC-1]=0;
    for(i = j ; i < MAXNN ; i++)
        line[i] = &(ws[MAXIFC-1]);
    listno = 0;
    setn(listno);
}

void slcalcEditConsFunc::getn(int n)
{
    int i, p;

    for (i = 0 ; i < 10 ; i++) {
        check[n+i]   = CheckBox[i]->isChecked();
        if (strcmp(line[n+i], LineEdit[i]->text())!=0) {
            p = ptr + strlen(line[n+i]) + 1;
            if (p < MAXIFC - 1) {
                edited[n+i] = true;
                line[n+i] = &(ws[ptr]);
                strcpy(line[n+i],  LineEdit[i]->text());
                ptr = p;
            }
        }
    }
}

slcalc::slcalc( QWidget* parent,  const char* name, WFlags fl )
    : slcalcBase( parent, name, fl )
{
// menu
    edit = new QPopupMenu( this );
    edit->setCaption("Edit");
    edit->insertItem( "Cu&t",   this, SLOT(EditCut()) );
    edit->insertItem( "&Copy",  this, SLOT(EditCopy()) );
    edit->insertItem( "&Paste", this, SLOT(EditPaste()) );
    edit->insertSeparator();
    edit->insertItem( "&Ans->Clipboard", this, SLOT(AnsCopy()) );

    options = new QPopupMenu( this );
    options->setCaption("Options");
    options->insertItem( "show &Log", this, SLOT(log()) );
    options->insertItem( "Edit &constants && functions", this, SLOT(editConsFunc()) );
    options->insertItem( "load &File", this, SLOT(LoadFile()) );
    options->insertSeparator();
    options->insertItem( "&Setting", this, SLOT(setting()) );

    help = new QPopupMenu( this );
    help->setCaption("Help");
    help->insertItem( "&Keyboard",  this, SLOT(keyhelp()) );
    help->insertItem( "O&perators", this, SLOT(operators()) );
    help->insertSeparator();
    help->insertItem( "&About", this, SLOT(about()) );

    menu = new QPEMenuBar( this );
    menu->insertItem( "&Options", options );
    menu->insertItem( "&Edit",    edit );
    menu->insertItem( "&Help",    help );

    topLayout->setMenuBar(menu);
    topLayout->activate();

    popupSetting      = new slcalcSetting( this );
    popupList         = new slcalcList( this );
    popupEditConsFunc = new slcalcEditConsFunc( this );
    popupFileNameDiag = new slcalcFileNameDiag(this);

// PushButton
    connect(PushButtonEnt,     SIGNAL(clicked()), this, SLOT(buttonEnt()));
    connect(PushButtonCls,     SIGNAL(clicked()), this, SLOT(buttonCls()));
    connect(PushButtonBs,      SIGNAL(clicked()), this, SLOT(buttonBs()));

    connect(PushButton0,       SIGNAL(clicked()), this, SLOT(button0()));
    connect(PushButton1,       SIGNAL(clicked()), this, SLOT(button1()));
    connect(PushButton2,       SIGNAL(clicked()), this, SLOT(button2()));
    connect(PushButton3,       SIGNAL(clicked()), this, SLOT(button3()));
    connect(PushButton4,       SIGNAL(clicked()), this, SLOT(button4()));
    connect(PushButton5,       SIGNAL(clicked()), this, SLOT(button5()));
    connect(PushButton6,       SIGNAL(clicked()), this, SLOT(button6()));
    connect(PushButton7,       SIGNAL(clicked()), this, SLOT(button7()));
    connect(PushButton8,       SIGNAL(clicked()), this, SLOT(button8()));
    connect(PushButton9,       SIGNAL(clicked()), this, SLOT(button9()));
    connect(PushButtonDot,     SIGNAL(clicked()), this, SLOT(buttonDot()));
    connect(PushButtonPlus,    SIGNAL(clicked()), this, SLOT(buttonPlus()));
    connect(PushButtonMinus,   SIGNAL(clicked()), this, SLOT(buttonMinus()));
    connect(PushButtonDiv,     SIGNAL(clicked()), this, SLOT(buttonDiv()));
    connect(PushButtonMul,     SIGNAL(clicked()), this, SLOT(buttonMul()));
    connect(PushButtonLp,      SIGNAL(clicked()), this, SLOT(buttonLp()));
    connect(PushButtonRp,      SIGNAL(clicked()), this, SLOT(buttonRp()));
    connect(PushButtonPow,     SIGNAL(clicked()), this, SLOT(buttonPow()));
    connect(PushButtonPercent, SIGNAL(clicked()), this, SLOT(buttonPercent()));
    connect(PushButtonEq,      SIGNAL(clicked()), this, SLOT(buttonEq()));
    connect(PushButtonEe,      SIGNAL(clicked()), this, SLOT(buttonEe()));
    connect(PushButtonComma,   SIGNAL(clicked()), this, SLOT(buttonComma()));
    connect(PushButtonSpace,   SIGNAL(clicked()), this, SLOT(buttonSpace()));

    connect(PushButtonRecall,     SIGNAL(clicked()), this, SLOT(recall()));
    connect(PushButtonBackrecall, SIGNAL(clicked()), this, SLOT(backrecall()));

// ListBox
    connect( ListBox1, SIGNAL(clicked(QListBoxItem*)), this, SLOT(selection1(QListBoxItem*)) );
    connect( ListBox2, SIGNAL(clicked(QListBoxItem*)), this, SLOT(selection2(QListBoxItem*)) );
    connect( ListBox1, SIGNAL(returnPressed(QListBoxItem*)), this, SLOT(selection1(QListBoxItem*)) );
    connect( ListBox2, SIGNAL(returnPressed(QListBoxItem*)), this, SLOT(selection2(QListBoxItem*)) );

// RadioButton
    connect( RadioButtonN, SIGNAL(clicked()), this, SLOT(rbuttonN()) );
    connect( RadioButtonF, SIGNAL(clicked()), this, SLOT(rbuttonF()) );
    connect( RadioButtonE, SIGNAL(clicked()), this, SLOT(rbuttonE()) );

    connect( RadioButtonRad, SIGNAL(clicked()), this, SLOT(abuttonR()) );
    connect( RadioButtonDeg, SIGNAL(clicked()), this, SLOT(abuttonD()) );
//
    connect( MultiLineEdit1, SIGNAL(keyPress(QKeyEvent *)), this, SLOT(keypressed(QKeyEvent *)) );

// set flag to clear text and buffer position
    g_bClear  = true;
    g_bBufCount = 0;

// read slcalc.conf
    ConfLoad(conf_file_name);

// set format and angle base
    rbuttonN();  // format n
    abuttonR();  // angle rad

// read previouse buffer
    if (g_sFlagBuffer) bufferLoad(g_sBufferFileName);

// set name of constants and functions 
    resetList();
//
    input();
}

slcalc::~slcalc()
{
// store buffer
    if (g_sFlagBuffer) bufferStore(g_sBufferFileName);
// store slcalc.conf
    ConfStore(conf_file_name);
}

/*  
 *  functions
 */

#define PREDC  3  // number of pre-defined constants
#define PREDF 13  // number of pre-defined functions

void slcalc::resetList()
{
    ListBox1->clear();
    ListBox1->insertItem("Ans",   -1);
    ListBox1->insertItem("M",     -1);
    ListBox1->insertItem("pi",    -1);

    ListBox2->clear();
    ListBox2->insertItem("abs",   -1);
    ListBox2->insertItem("sqrt",  -1);
    ListBox2->insertItem("exp",   -1);
    ListBox2->insertItem("ln",    -1);
    ListBox2->insertItem("log",   -1);
    ListBox2->insertItem("sin",   -1);
    ListBox2->insertItem("cos",   -1);
    ListBox2->insertItem("tan",   -1);
    ListBox2->insertItem("asin",  -1);
    ListBox2->insertItem("acos",  -1);
    ListBox2->insertItem("atan",  -1);
    ListBox2->insertItem("int",   -1);
    ListBox2->insertItem("round", -1);

    SetConstName();
}

// set name of constants and functions to listbox
void slcalc::SetConstName()
{
    char *name[MAXNN];
    int i, index[MAXNN];
    unsigned int j;

    int n = calc_.read_table(name, index);
    index[n] = -1;
    for (j=PREDC; j<ListBox1->count(); j++) {
        for (i=0; (strcmp(ListBox1->text(j),name[i])!=0)&&(i<n);i++);
        if (index[i]!=0) ListBox1->removeItem(j--);
    }
    for (j=PREDF; j<ListBox2->count(); j++) {
        for (i=0; (strcmp(ListBox2->text(j),name[i])!=0)&&(i<n);i++);
        if (index[i]<=0) ListBox2->removeItem(j--);
    }
    for(i = 0; i < n; i++)
        if (index[i] == 0) {
            for (j=PREDC; ((j<ListBox1->count())&&(strcmp(ListBox1->text(j), name[i])!=0)); j++);
            if (j==ListBox1->count()) ListBox1->insertItem(name[i], -1);
        } else if (index[i] > 0) {
            for (j=PREDF; ((j<ListBox2->count())&&(strcmp(ListBox2->text(j), name[i])!=0)); j++);
            if (j==ListBox2->count()) ListBox2->insertItem(name[i], -1);
        }
}

// check whether text must be cleared or not
bool slcalc::checkClear()
{
    int line, col;

    MultiLineEdit1->getCursorPosition(&line, &col);
    if ( ( col != 0 ) || ( !MultiLineEdit1->atEnd() ) )
        g_bClear = false;
    return g_bClear;
}

// put string on input text
void slcalc::key(QString s) 
{
    int line, col;

    if (checkClear()) {
        g_bClear = false;
        MultiLineEdit1->setText("");
    } else
        MultiLineEdit1->delMarkedRegion();
    MultiLineEdit1->getCursorPosition(&line, &col);
    MultiLineEdit1->insertAt(s, line, col, false);
    g_bBufCount = 0;
}

// load file and calculate
void slcalc::FileLoad(const char *filename)
{
    char s[MAXIFC], tx[MAXRC];
    int i;
    double x;

    QFile f(filename);
    if (!f.open(IO_ReadOnly)) return;
    for( i = 0 ; (!f.atEnd()) && (i<MAXIFC) ; i++ ) s[i] = f.getch();
    if ( i >= MAXIFC ) i = MAXIFC - 1;
    s[i] = 0;
    f.close();
    if (calc_.result(s, &x, tx)!=0) {
        strcpy(s,"File ");
        strcat(s,tx);
        LineEdit2->setText(s);
    } else
        LineEdit2->setText("File successfully loaded");
    SetConstName();
}

// store slcalc.conf
void slcalc::ConfStore(const char *fname)
{
    QString  s;

    if (!g_bCFedited) return;
    QFile f(fname);
    if (!f.open(IO_WriteOnly)) return;

    s = "formatN = " + g_sFormatN + "\n";
    f.writeBlock(s, s.length());    
    s = "formatF = " + g_sFormatF + "\n";
    f.writeBlock(s, s.length());    
    s = "formatE = " + g_sFormatE + "\n";
    f.writeBlock(s, s.length());    
    s = "storeBuffer = ";
    if (g_sFlagBuffer) s += "yes\n"; else s += "no\n";
    f.writeBlock(s, s.length());    
    s = "BufferStoreFileName = " + g_sBufferFileName + "\n";
    f.writeBlock(s, s.length());    

    f.close();
}

// load slcalc.conf
void slcalc::ConfLoad(const char *fname)
{
    char  s[MAXSC], t[MAXSC], u[MAXSC];

    QFile f(fname);
    if (!f.open(IO_ReadOnly)) return;

    while (!f.atEnd()){
        f.readLine(s, MAXSC-1);
        int i = 0;
        while (isspace(s[i])) i++;
        int j = 0;
        while ( (s[i]!='=') && (!isspace(s[i])) && (s[i]!=0) )
            t[j++] = toupper(s[i++]);
        t[j] = 0;
        while ((s[i]=='=')||(isspace(s[i]))) i++;
        j = 0;
        while ( (s[i]!='=') && (!isspace(s[i])) && (s[i]!=0) )
            u[j++] = s[i++];
        u[j] = 0;
        if (strcmp(t,"FORMATN")==0) {
            g_sFormatN = u;
        } else if (strcmp(t,"FORMATF")==0) {
            g_sFormatF = u;
        } else if (strcmp(t,"FORMATE")==0) {
            g_sFormatE = u;
        } else if (strcmp(t,"STOREBUFFER")==0) {
            g_sFlagBuffer = (toupper(u[0])=='Y');
        } else if (strcmp(t,"BUFFERSTOREFILENAME")==0) {
            g_sBufferFileName = u;
        }
    }
    f.close();
}

void slcalc::bufferStore(const char *fname)
{
    char s[MAXIFC];
    int i, n, no;

    QFile f(fname);
    if (!f.open(IO_WriteOnly)) return;

    no = 0;
    while ( buffer_.Get(no, s) && buffer_r.Get(no, s)) no++;
    no--;
    n = no;
    while ( buffer_.Get(no, s) ) {
        for(i=0; s[i]; i++) f.putch(s[i]);
        f.putch(0);
        no --;
    }

    f.putch(1);
    no = n;
    while ( buffer_r.Get(no, s) ) {
        for(i=0; s[i]; i++) f.putch(s[i]);
        f.putch(0);
        no --;
    }

    f.putch(1);
    calc_.list(s, MAXIFC);
    for(i=0; s[i]; i++) f.putch(s[i]);

    f.close();
}

void slcalc::bufferLoad(const char *fname)
{
    char s[MAXIFC], tx[MAXRC];
    int i = 0;
    double x;

    QFile f(fname);
    if (!f.open(IO_ReadOnly)) return;

    while (!f.atEnd()){
        i = 0;
        s[i] = f.getch();
        if (s[i]==1) break;
        while ((s[i])&&(!f.atEnd()))
            s[++i] = f.getch();
        s[++i] = 0;
        buffer_.Put(s);
    }

    if (s[i]==1)
    while (!f.atEnd()){
        i = 0;
        s[i] = f.getch();
        if (s[i]==1) break;
        while ((s[i])&&(!f.atEnd()))
            s[++i] = f.getch();
        s[++i] = 0;
        buffer_r.Put(s);
    }

    if (s[i]==1)
    while (!f.atEnd()){
        i = 0;
        s[i] = f.getch();
        if (s[i]==1) break;
        while ((s[i])&&(!f.atEnd()))
            if (s[i]=='\n') {
                s[i] = 0;
                calc_.result(s, &x, tx);
                i = 0;
                s[i] = f.getch();
            } else
                s[++i] = f.getch();
        s[++i] = 0;
        calc_.result(s, &x, tx);
    }

    f.close();
}

/*
 *  slots
 */

void slcalc::input()
{
    MultiLineEdit1->setFocus();
}

void slcalc::keypressed(QKeyEvent * k)
{
//
//    char temp[64];
//    sprintf(temp,"(%d,%d)",k->state(),k->key());
//    key(temp);
//
    bool keyFind = true;
    bool s = (k->state() & ShiftButton);
    bool a = (k->state() & AltButton);
    bool c = (k->state() & ControlButton);
    switch (k->key()) {
        case Key_X           : if(!a && !c) key(k->text()); else EditCut();   break;
        case Key_C           : if(!a && !c) key(k->text()); else EditCopy();  break;
        case Key_V           : if(!a && !c) key(k->text()); else EditPaste(); break;
        case Key_Z           : if(!a && !c) key(k->text()); break;
        case Key_Plus        : buttonPlus();                break;
        case Key_Minus       : buttonMinus();               break;
        case Key_BracketLeft :
        case Key_Asterisk    : buttonMul();                 break;
        case Key_AsciiCircum : buttonPow();                 break;
        case Key_Equal       : buttonEq();                  break;
        case Key_Slash       : buttonDiv();                 break;
        case Key_Period      : if (s) key("?");    else buttonDot();   break;
        case Key_Comma       : if (s) buttonDiv(); else buttonComma(); break;
        case Key_Space       : buttonSpace();               break;
        case Key_Return      : buttonEnt();                 break;
        case Key_Left        : MultiLineEdit1->cursorL(s);  break;
        case Key_Right       : MultiLineEdit1->cursorR(s);  break;
        case Key_Up          : MultiLineEdit1->cursorU(s);  break;
        case Key_Down        : MultiLineEdit1->cursorD(s);  break;
        case Key_Home        : recall();                    break;
        case Key_End         : backrecall();                break;
        case Key_PageUp      : buttonCls();                 break;
        case Key_PageDown    : MultiLineEdit1->nl();        break;
        case Key_BackSpace   : buttonBs();                  break;
        case Key_Delete      : MultiLineEdit1->dl();        break;

        case Key_F11         : menu->setFocus();            break;

        case 0x1051          : // power switch
        case Key_Tab         :
        case Key_Shift       :
        case Key_F22         :                              break;

        default              : keyFind = false;
    }
    if (keyFind) return;
    if (SLC700)
        switch (k->key()) {
            case Key_0           : if (s) ListBox2->setFocus(); else button0(); break;
            case Key_1           : if (s) rbuttonN();           else button1(); break;
            case Key_2           : if (s) rbuttonF();           else button2(); break;
            case Key_3           : if (s) rbuttonE();           else button3(); break;
            case Key_4           : if (s) ListBox1->setFocus(); else button4(); break;
            case Key_5           : if (s) buttonPercent();      else button5(); break;
            case Key_6           : if (s) abuttonR();           else button6(); break;
            case Key_7           : if (s) abuttonD();           else button7(); break;
            case Key_8           : if (s) buttonLp();           else button8(); break;
            case Key_9           : if (s) buttonRp();           else button9(); break;
            case 0xa2            : buttonLp();                  break;
            case 0xa3            : buttonRp();                  break;
            case Key_BracketRight: buttonPercent();             break;
            case Key_Backslash   : switch (g_bFormat) {
                                       case 0:  rbuttonF();     break;
                                       case 1:  rbuttonE();     break;
                                       case 2:  rbuttonN();     break;
                                       default: rbuttonN();
                                   }                            break;
            case 0x2018          : switch (g_bAng) {
                                       case 0:  abuttonD();     break;
                                       case 1:  abuttonR();     break;
                                       default: abuttonD();
                                   }                            break;
            case 0x2014          :
            case 0x2015          : ListBox1->setFocus();        break;
            case 0x2022          : ListBox2->setFocus();        break;
            case 0x2017          : AnsCopy();                   break;
            case Key_Semicolon   : key(">");                    break;
            case Key_Bar         : key("<");                    break;
            case 0x2016          : buttonPlus();                break;
            case 0x201a          : buttonMinus();               break;
            case 0xa5            : buttonDiv();                 break;
            case 0x2019          : MultiLineEdit1->nl();        break;

            default              : key(k->text());
        }
    else
        switch (k->key()) {
            case Key_0           : button0();  break;
            case Key_1           : button1();  break;
            case Key_2           : button2();  break;
            case Key_3           : button3();  break;
            case Key_4           : button4();  break;
            case Key_5           : button5();  break;
            case Key_6           : button6();  break;
            case Key_7           : button7();  break;
            case Key_8           : button8();  break;
            case Key_9           : button9();  break;
            case Key_ParenLeft   : buttonLp(); break;
            case Key_ParenRight  : buttonRp(); break;

            default              : key(k->text());
        }
}

// menu setting
void slcalc::setting()
{
    popupSetting->set(this);
    popupSetting->PushButtonCancel->setFocus();
    popupSetting->show( );
}

// menu view log
void slcalc::log()
{
    char sb[MAXIC], sr[MAXRC];

    int no = 0;
    ws[0] = 0;
    while ( (buffer_.Get(no, sb)) && (buffer_r.Get(no, sr)) ) {
        if ( strlen(ws) + strlen(sb) + strlen(sr) + 7 >= MAXIFC ) break;
        strcat(ws, sb);
        strcat(ws, "\nAns=");
        strcat(ws, sr);
        strcat(ws, "\n\n");
        no ++;
    }
    popupList->list(this, ws);
    popupList->TextBrowser1->setFocus();
    popupList->show( );
}

// menu edit constants & functions
void slcalc::editConsFunc()
{
    calc_.list(ws, MAXIFC);
    popupEditConsFunc->set(this);
    popupEditConsFunc->PushButtonCancel->setFocus();
    popupEditConsFunc->show( );
}

// menu edit load file
void slcalc::LoadFile()
{
    QString FileName;

    popupFileNameDiag->list(this, "./");
    popupFileNameDiag->ListBox1->setFocus();
    popupFileNameDiag->show( );
}

// menu help keyboard
void slcalc::keyhelp()
{
    QMessageBox::about( this, "keyboard",
                       "format toggle :Fn+F\n"
                       "angle  toggle :Fn+S\n"
                       "constant list :Fn+O or Q\n"
                       "function list :Fn+K\n"
                       "clear      :Fn+cursorUp\n"
                       "new line   :Fn+cursorDown\n"
                       "recall     :Fn+cursorLeft\n"
                       "backrecall :Fn+cursorRight\n"
                      );
}

// menu help operators
void slcalc::operators()
{
    QMessageBox::about( this, "operators",
                       " 1) (  )\n"
                       " 2) not\n"
                       " 3) ^\n"
                       " 4) * / %\n"
                       " 5) + -\n"
                       " 6) < > <= >=\n"
                       " 7) eq ne\n"
                       " 8) and\n"
                       " 9) or\n"
                       "10) while if then else\n"
                       "11) =\n"
                      );
}

// menu about
void slcalc::about()
{
    QMessageBox::about( this, "slcalc",
                       "ver 1.2.1  15th November 2004\n"
                       "GPL freeware\n"
                       "http://\n"
                       "homepage3.nifty.com/cam/slcalc.htm"
                      );
}

// menu cut
void slcalc::EditCut()
{
    MultiLineEdit1->cut();
    input();
}

// menu copy
void slcalc::EditCopy()
{
    MultiLineEdit1->copy();
    input();
}

// menu paste
void slcalc::EditPaste()
{
    MultiLineEdit1->paste();
    input();
}

// menu Ans->Clipboard
void slcalc::AnsCopy()
{
    LineEdit2->selectAll();
    LineEdit2->copy();
    LineEdit2->deselect();
    input();
}

// click constant litbox
void slcalc::selection1(QListBoxItem * item)
{
    if (item)
        key(item->text());
    input();
}

// click fubctions listbox
void slcalc::selection2(QListBoxItem* item)
{
    int line, col;

    if (item) {
        key(item->text()+"(");
        MultiLineEdit1->getCursorPosition(&line, &col);
        MultiLineEdit1->insertAt(")", line, col, false);
        MultiLineEdit1->setCursorPosition(line, col);
    }
    input();
}

// calculate
void slcalc::buttonEnt()
{
    char s[MAXIC], tx[MAXRC], f[MAXFC];
    int  i, j;

// set result format
    switch(g_bFormat) {
        case 0:  strcpy(f, g_sFormatN); break;
        case 1:  strcpy(f, g_sFormatF); break;
        case 2:  strcpy(f, g_sFormatE); break;
        default: strcpy(f, "%g");
    }
    QString t = MultiLineEdit1->text();
    if (t.length() >= MAXIC) t = t.left(MAXIC - 1);
    strcpy(s, t);

// remove last empty lines
    i = 0;
    while (s[i]) i++;
    i--;
    while (isspace(s[i])) i--;
    s[++i]=0;

// buffer store
    buffer_.Put(s);

// calculation
    i = calc_.result(s, &g_bX, tx, f);
    if ((i==1)||(i==2)) {
       calc_.list(ws, MAXIFC);
       calc_.clear();
       for(i=j=0; ws[i]; i++)
          if (ws[i]=='\n') {
             ws[j] = 0;
             calc_.result(ws, &g_bX, tx, f);
             j=0;
          } else ws[j++] = ws[i];
       ws[j] = 0;
       calc_.result(ws, &g_bX, tx, f);
       i = calc_.result(s, &g_bX, tx, f);
    }
    g_bError = (i != 0);
    buffer_r.Put(tx);
    LineEdit2->setText(tx);
    SetConstName();

// cursol set
    int line = MultiLineEdit1->numLines()-1;
    t = MultiLineEdit1->textLine(line);
    int col = t.length();
    if ( col != 0 ) {
        line ++;
        MultiLineEdit1->insertLine("",line);
        col = 0;
    }
    MultiLineEdit1->setCursorPosition(line, col);

// set buffer count
    g_bBufCount = 1;

// text clear at next input        
    g_bClear = true;
}

// clear input text
void slcalc::buttonCls()
{
    MultiLineEdit1->setText("");
    g_bBufCount = 0;
}

// backspace input text
void slcalc::buttonBs()
{
    MultiLineEdit1->bs();
    g_bClear = false;
    g_bBufCount = 0;
}

// click radio button format n
void slcalc::rbuttonN()
{
    RadioButtonN->setChecked(true);
    RadioButtonF->setChecked(false);
    RadioButtonE->setChecked(false);
    g_bFormat = 0;
    if (!g_bError) {
        char tx[MAXRC], f[MAXFC];

        strcpy(f, g_sFormatN);
        calc_.sprintfn(tx, f, g_bX);
        LineEdit2->setText(tx);
    }
}

// click radio button format f
void slcalc::rbuttonF()
{
    RadioButtonN->setChecked(false);
    RadioButtonF->setChecked(true);
    RadioButtonE->setChecked(false);
    g_bFormat = 1;
    if (!g_bError) {
        char tx[MAXRC], f[MAXFC];

        strcpy(f, g_sFormatF);
        calc_.sprintfn(tx, f, g_bX);
        LineEdit2->setText(tx);
    }
}

// click radio button format e
void slcalc::rbuttonE()
{
    RadioButtonN->setChecked(false);
    RadioButtonF->setChecked(false);
    RadioButtonE->setChecked(true);
    g_bFormat = 2;
    if (!g_bError) {
        char tx[MAXRC], f[MAXFC];

        strcpy(f, g_sFormatE);
        calc_.sprintfn(tx, f, g_bX);
        LineEdit2->setText(tx);
    }
}

// click radio button angle base radian
void slcalc::abuttonR()
{
    RadioButtonRad->setChecked(true);
    RadioButtonDeg->setChecked(false);
    g_bAng = 0;
    calc_.set_ang(0);
}

// click radio button angle base degree
void slcalc::abuttonD()
{
    RadioButtonRad->setChecked(false);
    RadioButtonDeg->setChecked(true);
    g_bAng = 1;
    calc_.set_ang(1);
}

// click pushbutton recall
void slcalc::recall()
{
    char  s[MAXIC];

    if (buffer_.Get(g_bBufCount, s)) {
        MultiLineEdit1->setText(s);
        g_bBufCount ++;

        int line = MultiLineEdit1->numLines()-1;
        QString t = MultiLineEdit1->textLine(line);
        int col = t.length();
        MultiLineEdit1->setCursorPosition(line, col);
    }
    g_bClear = false;
}

// click pushbutton backrecall
void slcalc::backrecall()
{
    char  s[MAXIC];

    if (buffer_.Get(g_bBufCount-2, s)) {
        MultiLineEdit1->setText(s);
        g_bBufCount --;

        int line = MultiLineEdit1->numLines()-1;
        QString t = MultiLineEdit1->textLine(line);
        int col = t.length();
        MultiLineEdit1->setCursorPosition(line, col);
    }
    g_bClear = false;
}

// simply add charceters to input text

void slcalc::button0()
{
    key("0");
}

void slcalc::button1()
{
    key("1");
}

void slcalc::button2()
{
    key("2");
}

void slcalc::button3()
{
    key("3");
}

void slcalc::button4()
{
    key("4");
}

void slcalc::button5()
{
    key("5");
}

void slcalc::button6()
{
    key("6");
}

void slcalc::button7()
{
    key("7");
}

void slcalc::button8()
{
    key("8");
}

void slcalc::button9()
{
    key("9");
}

void slcalc::buttonDot()
{
    key(".");
}

void slcalc::buttonPlus()
{
    if (checkClear())
        key("Ans+");
    else
        key("+");
}

void slcalc::buttonMinus()
{
    if (checkClear())
        key("Ans-");
    else
        key("-");
}

void slcalc::buttonDiv()
{
    if (checkClear())
        key("Ans/");
    else
        key("/");
}

void slcalc::buttonMul()
{
    if (checkClear())
        key("Ans*");
    else
        key("*");
}

void slcalc::buttonLp()
{
    key("(");
}

void slcalc::buttonRp()
{
    key(")");
}

void slcalc::buttonPow()
{
    if (checkClear())
        key("Ans^");
    else
        key("^");
}

void slcalc::buttonPercent()
{
    if (checkClear())
        key("Ans%");
    else
        key("%");
}

void slcalc::buttonEq()
{
    key("=");
}

void slcalc::buttonEe()
{
    key("e");
}

void slcalc::buttonComma()
{
    key(",");
}

void slcalc::buttonSpace()
{
//    g_bClear = false;
    key(" ");
}
