// file slcalc.cpp
//   copyright  nakazato@phy.cam.ac.uk
//   1st Feb. 2003
//      first version
//   8th Feb. 2003
//      cut& paste buttons are added
//      slcalc::backspace is rewritten to avoid flashing
//      cursol positon and action to marked text are unified
//   15th Feb. 2003
//      menu (edit, options, help) is added
//   23rd Feb. 2003
//      add edit menu constants and functions
//      install keyboard input method
//   26th Feb. 2003
//      change qmenubar to qpemenubar
//   28th Feb. 2003
//      unify VGA, VGA view style, QVGA modes
//   23rd July 2003
//      add options menu load file

#ifndef slcalc_H
#define slcalc_H
#include "slcalcbase.h"
#include "slcalcSet.h"
#include "slcalcL.h"
#include "slcalcECF.h"
#include "calc.h"
#include "buffer.h"
#include "getFileNameDiag.h"
#include <qlistbox.h>
#include <qpe/qpemenubar.h>

#define MAXNN    100 // max name number
#define MAXIC    512 // max input    characters
#define MAXRC     64 // max result   characters
#define MAXFC     16 // max format   characters
#define MAXSC    128 // max standard characters
#define MAXIFC 16384 // max input file characters

class slcalc : public slcalcBase
{ 
    Q_OBJECT

public:
    slcalc( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~slcalc();
    void resetList();
    void FileLoad(const char *filename);
    void key(QString s);

private:
    buffer buffer_;
    buffer buffer_r;
    QPEMenuBar* menu;
    QPopupMenu* edit;
    QPopupMenu* options;
    QPopupMenu* help;

    void SetConstName();
    bool checkClear();
    void ConfStore(const char *fname);
    void ConfLoad(const char *fname);
    void bufferStore(const char *fname);
    void bufferLoad(const char *fname);

private slots:
    void input();
    void keypressed(QKeyEvent *);
    void setting();
    void log();
    void editConsFunc();
    void LoadFile();
    void keyhelp();
    void operators();
    void about();

    void EditCut();
    void EditCopy();
    void EditPaste();
    void AnsCopy();

    void selection1(QListBoxItem *item);
    void selection2(QListBoxItem *item);

    void buttonEnt();
    void buttonCls();
    void buttonBs();

    void rbuttonN();
    void rbuttonF();
    void rbuttonE();
    void abuttonR();
    void abuttonD();

    void recall();
    void backrecall();

    void button0();
    void button1();
    void button2();
    void button3();
    void button4();
    void button5();
    void button6();
    void button7();
    void button8();
    void button9();
    void buttonDot();
    void buttonPlus();
    void buttonMinus();
    void buttonDiv();
    void buttonMul();
    void buttonLp();
    void buttonRp();
    void buttonPow();
    void buttonPercent();
    void buttonEq();
    void buttonEe();
    void buttonComma();
    void buttonSpace();
};

class slcalcSetting : public slcalcSet
{
    Q_OBJECT

public:
    slcalcSetting( QWidget* parent = 0, const char* name = 0 );
    ~slcalcSetting();
    void set(slcalc * parent);

private:
    slcalc * popupParent;

private slots:
    void buttonOK();
    void buttonCancel();
};

class slcalcList : public slcalcL
{
    Q_OBJECT

public:
    slcalcList( QWidget* parent = 0, const char* name = 0 );
    ~slcalcList();
    void list(slcalc * parent, char *s);

private:
    slcalc * popupParent;

private slots:
    void closelist();
};

class slcalcEditConsFunc : public slcalcECF
{
    Q_OBJECT

public:
    slcalcEditConsFunc( QWidget* parent = 0, const char* name = 0 );
    ~slcalcEditConsFunc();
    void set(slcalc * parent);

private:
    bool check[MAXNN];
    bool edited[MAXNN];
    int  index[MAXNN];
    char *line[MAXNN];
    int  listno;
    int  ptr;
    slcalc * popupParent;
    void setn(int n);
    void getn(int n);

private slots:
    void buttonOK();
    void buttonCancel();
    void buttonPrev();
    void buttonNext();
};

class slcalcFileNameDiag : public getFileNameDiag
{
    Q_OBJECT

public:
    slcalcFileNameDiag( QWidget* parent = 0, const char* name = 0 );
    ~slcalcFileNameDiag();
    void list(slcalc * parent, char *s);

private:
    slcalc * popupParent;

private slots:
    void OK_selected();
    void Cancel_selected();
};

#endif // slcalc_H
